function ControlApp() {
    this.alertElement = null;
    this.alertTimeout = null;
    this.toplevel = null;
    this.home = null;
}

ControlApp.prototype = {
    initilialize: function(ifaces) {
        this.ifaces = ifaces;

        // Get needed elements
        this.toplevel = document.getElementById("toplevel");
        this.home = document.getElementById("home");
        this.title = document.getElementById("title");

        // Initialize home button
        this.home.addEventListener("click", () => this.onHomeClick());

        // Initialize error dialog
        this.alertElement = document.getElementById("alert-text");

        // Initialize player listener
        ifaces.player.getState().then((state) => this.onPlayerStateChange(state));
        ifaces.player.addListener((event) => this.onPlayerEvent(event));

        // Load config
        this.getApplicationConfiguration()
            .then((config) => {
                this.title.innerText = config.title;
                this.createVideoContainer(config);
            })
            .catch(() => this.displayError("Erreur lors du chargement de la configuration (config.json)"));
    },

    createVideoContainer: function(config) {
        var videos = config.videos;
        var container = document.getElementById("video-grid");

        if (videos && videos.length) {
            for (var i = 0 ; i < videos.length ; i++) {
                var current = videos[i];
                container.appendChild(this.createVideoItem(current));
            }
        } else {
            this.displayError("Aucune vidéo définie !");
        }
    },

    createVideoItem: function(params) {
        var cell = document.createElement("div");
        cell.className = "video-cell";


        if (!params.empty) {
            var button = document.createElement("div");
            button.className = "video-button";
            button.style.backgroundImage = "url(\"images/" + params.image + "\")"

            cell.appendChild(button);

            var veil = document.createElement("div");
            button.appendChild(veil);

            var title = document.createElement("div");
            title.innerText = params.title;
            veil.appendChild(title);

            var folder = params.folder;
            button.addEventListener("click", () => this.onVideoClick(folder));
        }

        return cell;
    },

    onHomeClick: function() {
        this.ifaces.player.stop();
    },

    onPlayerEvent: function(event) {
        if (event.detail.type == "state-change") {
            this.onPlayerStateChange(event.detail.state);
        }
    },

    onPlayerStateChange: function(state) {
        switch (state) {
            case 'playing':
            case 'paused':
                this.toplevel.classList.add("playback");
                break;

            default:
                this.toplevel.classList.remove("playback");
                break;
        }
    },

    onVideoClick: function(folder) {
        console.log("Play folder " + folder);
        this.ifaces.player.playFolder(folder)
            .catch(() => this.displayError("Erreur lors de l'envoi de la commande de lecture du dossier " + folder));
    },

    getApplicationConfiguration: function() {
        return new Promise(function(resolve, reject) {
            var xhr = new XMLHttpRequest();
            xhr.addEventListener("readystatechange", function() {
                if(this.readyState === 4) {
                    if (xhr.status === 200) {
                        try {
                            var response = JSON.parse(this.responseText);
                            resolve(response);
                        } catch(e) {
                            reject(e);
                        }
                    } else {
                        reject("Invalid HTTP response status " + xhr.status);
                    }
                }
            });
            console.log("Getting config.json (you can edit this file with your settings)...");
            xhr.open("GET", "config.json");
            xhr.send();
        });
    },

    /**
     * Display an error message during a short period of time
     * @param {*} message
     */
    displayError: function(message) {
        console.log("Erreur: " + message);
        if (this.alertTimeout !== null) {
            window.clearTimeout(this.alertTimeout);
            this.alertTimeout = null;
        }

        this.alertElement.classList.add("show");
        this.alertElement.innerText = message;

        this.alertTimeout = window.setTimeout(() => {
            this.alertElement.classList.remove("show");
            this.alertTimeout = null;
        }, 15000);
    }
};



window.addEventListener('load', function() {
    IDAL.getControlInterfaces()
        .then(function(ifaces) {
            var app = new ControlApp();
            app.initilialize(ifaces);
        })

});